@@ -3,8 +3,8 @@ import os
 from functools import lru_cache
 
 import numpy as np
-from pygc import great_circle
-from skyfield.api import Topos, Star, Angle
+from skyfield.api import Star, Angle, iers2010
+from skyfield.toposlib import _ltude
 from skyfield.relativity import add_aberration, add_deflection
 from skyfield.earthlib import compute_limb_angle
 from skyfield.positionlib import Apparent
@@ -15,6 +15,11 @@ if 'SATSIM_SKYFIELD_LOAD_DIR' in os.environ:
 else:
     from skyfield.api import load
 
+from astropy.coordinates import SkyCoord, ICRS, ITRS
+from astropy.time import Time
+from astropy.coordinates import cartesian_to_spherical
+from astropy import units as u
+
 from satsim.geometry.wcs import get_min_max_ra_dec, get_wcs
 
 
@@ -23,47 +28,6 @@ SATSIM_MOON = None
 SATSIM_SUN = None
 
 
-class GreatCircle(object):
-    """ A great circle propagated skyfield object."""
-
-    def __init__(self, az, el, heading, velocity, t, observer):
-        """ Constructor.
-
-        Args:
-            az: `float`, starting azimuth or longitude in degrees
-            el: `float`, starting elevation or latitude in degrees
-            heading: `float`, direction in degrees. 0 degrees is north heading
-            velocity: `float`, velocity in degrees per second
-            t: `object`, skyfield time
-            observer: `object`, frame of reference, if `None`, reference is barycenter
-
-        Returns:
-            A `Topos` Skyfield object on the planet Earth
-        """
-        self.az = az
-        self.el = el
-        self.heading = heading
-        self.velocity = velocity
-        self.t = t
-        self.R = 57.29577951308232
-        self.target = None
-        self.observer = observer
-
-    def _observe_from_bcrs(self, observer):
-        t = (observer.t - self.t) * 86400
-        d = t * self.velocity
-        gc = great_circle(distance=d, azimuth=self.heading, latitude=self.el, longitude=self.az, rmajor=self.R, rminor=self.R)
-        if self.observer is not None:
-            ra, dec, _, _, _, _ = get_los_azel(self.observer, gc['longitude'], gc['latitude'], observer.t)
-        else:
-            ra = gc['longitude']
-            dec = gc['latitude']
-
-        s = Star(ra=Angle(degrees=ra), dec=Angle(degrees=dec), parallax_mas=1e-16)
-        vector, vel, t, light_time = s._observe_from_bcrs(observer)
-        return vector, vel, t, None
-
-
 def load_earth():
     """Loads a Skyfield Earth object using the planetary and lunar ephemeris,
     DE 421. The Earth object is loaded once and cached as a singleton in the
@@ -118,13 +82,14 @@ def load_sun():
     return SATSIM_SUN
 
 
-def create_topocentric(lat, lon):
+def create_topocentric(lat, lon, alt=0):
     """Create a Skyfield topocentric object which represents the location of a
     place on the planet Earth.
 
     Args:
         lat: `string or float`, latitude in degrees
         lon: `string or float`, longitude in degrees
+        alt: `float`, altitude in km
 
     Returns:
         A `Topos` Skyfield object on the planet Earth
@@ -132,7 +97,17 @@ def create_topocentric(lat, lon):
 
     earth = load_earth()
 
-    return earth + Topos(lat, lon)
+    if isinstance(lat, str):
+        lat = _ltude(lat, 'latitude', 'N', 'S')
+
+    if isinstance(lon, str):
+        lon = _ltude(lon, 'longitude', 'E', 'W')
+
+    topo = iers2010.latlon(latitude_degrees=lat,
+                           longitude_degrees=lon,
+                           elevation_m=alt * 1000)
+
+    return earth + topo
 
 
 def apparent(p, deflection=False, aberration=True):
@@ -182,7 +157,7 @@ def apparent(p, deflection=False, aberration=True):
         add_deflection(target_au, bcrs_position,
                        p._ephemeris, t, include_earth_deflection)
 
-    if aberration:
+    if aberration and p.light_time is not None:
         add_aberration(target_au, bcrs_velocity, p.light_time)
 
     apparent = Apparent(target_au, None, t, p.center, p.target)
@@ -191,8 +166,38 @@ def apparent(p, deflection=False, aberration=True):
     return apparent
 
 
+def _apply_stellar_aberration(observer, ra, dec, az, el, t, deflection):
+    """Apply stellar aberration to RA/Dec coordinates.
+
+    Args:
+        observer: Skyfield observer object
+        ra: Right ascension angle object
+        dec: Declination angle object
+        az: Azimuth angle object
+        el: Elevation angle object
+        t: Skyfield time object
+        deflection: Boolean for deflection correction
+
+    Returns:
+        Tuple of corrected (ra, dec, el, az) values
+    """
+    star = Star(ra=ra, dec=dec)
+    icrf_los = apparent(observer.at(t).observe(star), deflection, True)
+    sa_ra, sa_dec, sa_d = icrf_los.radec()
+    sa_el, sa_az, sa_d = icrf_los.altaz()
+
+    # Adjust the apparent RA and Dec based on stellar aberration
+    # Convert to degrees for arithmetic operations
+    apparent_ra = Angle(degrees=2 * ra.degrees - sa_ra.degrees)
+    apparent_dec = Angle(degrees=2 * dec.degrees - sa_dec.degrees)
+    apparent_el = Angle(degrees=2 * el.degrees - sa_el.degrees)
+    apparent_az = Angle(degrees=2 * az.degrees - sa_az.degrees)
+
+    return apparent_ra, apparent_dec, apparent_el, apparent_az
+
+
 @lru_cache(maxsize=32)
-def get_los(observer, target, t, deflection=False, aberration=True):
+def get_los(observer, target, t, deflection=False, aberration=True, stellar_aberration=False):
     """Get the apparent line of sight vector from an observer and target in
     right ascension (RA) and declination (Dec).
 
@@ -201,7 +206,8 @@ def get_los(observer, target, t, deflection=False, aberration=True):
         target: `object`, target as a Skyfield object
         t: `object`, skyfield time
         deflection: `boolean`, enable deflection adjustment
-        aberration: `boolean`, enable aberration of light adjustment
+        aberration: `boolean`, enable aberration of light adjustment for the target
+        stellar_aberration: `boolean`, enable stellar aberration adjustment (apparent)
 
     Returns:
         A `tuple`, containing:
@@ -213,15 +219,22 @@ def get_los(observer, target, t, deflection=False, aberration=True):
             icrf_los: LOS as skyfield ICRF object
     """
 
-    icrf_los = apparent(observer.at(t).observe(target), deflection, aberration)
+    if deflection or aberration:
+        icrf_los = apparent(observer.at(t).observe(target), deflection, aberration)
+    else:
+        icrf_los = (target - observer).at(t)
+
     ra, dec, d = icrf_los.radec()
     el, az, d = icrf_los.altaz()
 
+    if stellar_aberration:
+        ra, dec, el, az = _apply_stellar_aberration(observer, ra, dec, az, el, t, deflection)
+
     return ra._degrees, dec._degrees, d.km, az._degrees, el._degrees, icrf_los
 
 
 @lru_cache(maxsize=32)
-def get_los_azel(observer, az, el, t, deflection=False, aberration=True):
+def get_los_azel(observer, az, el, t, deflection=False, aberration=True, stellar_aberration=False):
     """Get the apparent line of sight vector from an observer based on topocentric
     az and el.
 
@@ -244,9 +257,63 @@ def get_los_azel(observer, az, el, t, deflection=False, aberration=True):
     icrf_los = apparent(observer.at(t).from_altaz(alt_degrees=el, az_degrees=az), deflection, aberration)
     ra, dec, d = icrf_los.radec()
 
+    if stellar_aberration:
+        # Convert float values to Angle objects for stellar aberration calculation
+        from skyfield.units import Angle
+        el_angle = Angle(degrees=el)
+        az_angle = Angle(degrees=az)
+        ra, dec, el_angle, az_angle = _apply_stellar_aberration(observer, ra, dec, az_angle, el_angle, t, deflection)
+        el = el_angle.degrees
+        az = az_angle.degrees
+
     return ra._degrees, dec._degrees, d.km, az, el, icrf_los
 
 
+def get_analytical_los(observer, target, t, frame="observer"):
+    """Return a line of sight in the specified frame for analytical observations.
+
+    Args:
+        observer: `object`, observer as a Skyfield object
+        target: `object`, target as a Skyfield object
+        t: `object`, skyfield time
+        frame: `string`, one of ``barycentric``, ``geocentric`` or ``observer``
+
+    Returns:
+        A `tuple`, containing:
+            ra: right ascension in degrees
+            dec: declination in degrees
+            d: distance between observer and target in km
+    """
+
+    frame = frame.lower()
+
+    # compute light time using apparent LOS from observer to target
+    icrf_los_abr = observer.at(t).observe(target)
+    lt = icrf_los_abr.light_time
+
+    try:
+        if frame == "barycentric":
+            icrf_los = target.at(t - lt) - observer.at(t)
+        elif frame == "geocentric":
+            earth = load_earth()
+            observer_gc = observer - earth
+            target_gc = target - earth
+            icrf_los = target_gc.at(t - lt) - observer_gc.at(t)
+        elif frame == "observer":  # observer frame
+            observer_oc = observer - observer
+            target_oc = target - observer
+            icrf_los = target_oc.at(t - lt) - observer_oc.at(t)
+        else:
+            raise ValueError(f"Unknown frame: {frame}")
+    except Exception:
+        # Handle cases where the observer or target does not support the requested frame
+        icrf_los = icrf_los_abr
+
+    ra, dec, d = icrf_los.radec()
+
+    return ra._degrees, dec._degrees, d.km
+
+
 def query_by_los(height, width, y_fov, x_fov, ra, dec, t0, observer, targets=[], rot=0, pad_mult=0, origin='center', offset=[0,0]):
     """Return objects that are within the minimum and maximum RA and declination
     bounds of the observer's focal plan array with padding, `pad_mult`.
@@ -361,7 +428,8 @@ def gen_track_from_wcs(height, width, wcs, observer, targets, t0, tt, origin='ce
     return rr, cc, (r1 - r0) / exposure_time, (c1 - c0) / exposure_time
 
 
-def wcs_from_observer_rate(height, width, y_fov, x_fov, observer, t0, tt, rot, track):
+def wcs_from_observer_rate(height, width, y_fov, x_fov, observer, t0, tt, rot, track,
+                           deflection=False, aberration=True, stellar_aberration=False):
     """Calculate the world coordinate system (WCS) transform from the `observer`
     at times `tt` while rate tracking the object, `track`.
 
@@ -381,13 +449,21 @@ def wcs_from_observer_rate(height, width, y_fov, x_fov, observer, t0, tt, rot, t
     """
     wsc = []
     for t in tt:
-        [ra0,dec0,d0,az0,el0,los0] = get_los(observer, track, t)
+        [ra0, dec0, d0, az0, el0, los0] = get_los(
+            observer,
+            track,
+            t,
+            deflection=deflection,
+            aberration=aberration,
+            stellar_aberration=stellar_aberration,
+        )
         wcs0 = get_wcs(height, width, y_fov / height, x_fov / width, ra0, dec0, rot)
         wsc.append(wcs0)
     return wsc
 
 
-def wcs_from_observer_fixed(height, width, y_fov, x_fov, observer, tt, rot, az, el):
+def wcs_from_observer_fixed(height, width, y_fov, x_fov, observer, tt, rot, az, el,
+                            deflection=False, aberration=True, stellar_aberration=False):
     """Calculate the world coordinate system (WCS) transform from the `observer`
     at a fixed pointing position based on azimuth and elevation.
 
@@ -406,14 +482,23 @@ def wcs_from_observer_fixed(height, width, y_fov, x_fov, observer, tt, rot, az,
         A `object`: WCS transform
     """
     wsc = []
-    for t,a,e in zip(tt,az,el):
-        [ra0,dec0,d0,az0,el0,los0] = get_los_azel(observer, a, e, t)
+    for t, a, e in zip(tt, az, el):
+        [ra0, dec0, d0, az0, el0, los0] = get_los_azel(
+            observer,
+            a,
+            e,
+            t,
+            deflection=deflection,
+            aberration=aberration,
+            stellar_aberration=stellar_aberration,
+        )
         wcs0 = get_wcs(height, width, y_fov / height, x_fov / width, ra0, dec0, rot)
         wsc.append(wcs0)
     return wsc
 
 
-def wcs_from_observer_sidereal(height, width, y_fov, x_fov, observer, t0, tt, rot, track):
+def wcs_from_observer_sidereal(height, width, y_fov, x_fov, observer, t0, tt, rot, track,
+                               deflection=False, aberration=True, stellar_aberration=False):
     """Calculate the world coordinate system (WCS) transform from the `observer`
     at times `tt` while sidereal tracking the object, `track`.
 
@@ -432,7 +517,14 @@ def wcs_from_observer_sidereal(height, width, y_fov, x_fov, observer, t0, tt, ro
         A `object`: WCS transform
     """
     wsc = []
-    [ra0,dec0,d0,az0,el0,los0] = get_los(observer, track, t0)
+    [ra0, dec0, d0, az0, el0, los0] = get_los(
+        observer,
+        track,
+        t0,
+        deflection=deflection,
+        aberration=aberration,
+        stellar_aberration=stellar_aberration,
+    )
     wcs0 = get_wcs(height, width, y_fov / height, x_fov / width, ra0, dec0, rot)
     for t in tt:
         wsc.append(wcs0)
@@ -440,7 +532,8 @@ def wcs_from_observer_sidereal(height, width, y_fov, x_fov, observer, t0, tt, ro
 
 
 def gen_track(height, width, y_fov, x_fov, observer, track, satellites, brightnesses, t0, tt, rot=0, pad_mult=0, track_type='rate',
-              offset=[0,0], flipud=False, fliplr=False, az=None, el=None):
+              offset=[0, 0], flipud=False, fliplr=False, az=None, el=None,
+              deflection=False, aberration=True, stellar_aberration=False):
     """Generates a list of pixel coordinates from the observing focal plane
     array to each satellite in the list, `satellites`. Track mode can be either
     `rate` or `sidereal`.
@@ -464,6 +557,9 @@ def gen_track(height, width, y_fov, x_fov, observer, track, satellites, brightne
         fliplr: `boolean`, flip array in left/right direction
         az: `float`, azimuth in degrees for `fixed` tracking
         el: `float`, elevation in degrees for `fixed` tracking
+        deflection: `bool`, include gravitational deflection in astrometry
+        aberration: `bool`, include light transit time effects in astrometry
+        stellar_aberration: `bool`, include stellar aberration in astrometry
 
     Returns:
         A `tuple`, containing:
@@ -473,12 +569,52 @@ def gen_track(height, width, y_fov, x_fov, observer, track, satellites, brightne
             dc: `float`, star motion along the columns in pixel per second
             b: `float`, list of brightnesses of each target in Mv
     """
+    # note wcs are based on apparent position of the tracked target
     if track_type == 'rate':
-        wcs = wcs_from_observer_rate(height, width, y_fov, x_fov, observer, t0, tt, rot, track)
+        wcs = wcs_from_observer_rate(
+            height,
+            width,
+            y_fov,
+            x_fov,
+            observer,
+            t0,
+            tt,
+            rot,
+            track,
+            deflection=deflection,
+            aberration=aberration,
+            stellar_aberration=stellar_aberration,
+        )
     elif track_type == 'fixed':
-        wcs = wcs_from_observer_fixed(height, width, y_fov, x_fov, observer, tt, rot, az, el)
+        wcs = wcs_from_observer_fixed(
+            height,
+            width,
+            y_fov,
+            x_fov,
+            observer,
+            tt,
+            rot,
+            az,
+            el,
+            deflection=deflection,
+            aberration=aberration,
+            stellar_aberration=stellar_aberration,
+        )
     else:
-        wcs = wcs_from_observer_sidereal(height, width, y_fov, x_fov, observer, t0, tt, rot, track)
+        wcs = wcs_from_observer_sidereal(
+            height,
+            width,
+            y_fov,
+            x_fov,
+            observer,
+            t0,
+            tt,
+            rot,
+            track,
+            deflection=deflection,
+            aberration=aberration,
+            stellar_aberration=stellar_aberration,
+        )
 
     b = np.asarray(brightnesses)
     visible = satellites
@@ -510,6 +646,23 @@ def angle_between(observer, object_a, object_b, t):
     return a.separation_from(b).degrees
 
 
+def distance_between(object_a, object_b, t):
+    """Calculate the distance between `object_a` and `object_b` at time `t`.
+
+    Args:
+        object_a: `object`, Skyfield object
+        object_b: `object`, Skyfield object
+        t: `Time`, Skyfield `Time`
+
+    Returns:
+        A `float`, distance in km
+    """
+    a = object_a.at(t)
+    b = object_b.at(t)
+
+    return (a - b).distance().km
+
+
 def angle_from_los(observer, object_a, ra, dec, t):
     """Calculate the angle between `observer` to `object_a` and `observer`
     to a line of sight in `ra` and `dec`. Angle is returned in degrees.
@@ -529,29 +682,73 @@ def angle_from_los(observer, object_a, ra, dec, t):
     return angle_between(observer, s, object_a, t)
 
 
-def lambertian_sphere_to_mv(albedo, distance, radius, phase_angle):
-    """Applies lambertian sphere approximation to convert target brightness
-    to visual magnitudes based on sun brightness of -26.74.
+def eci_to_ecr(time, ra, dec, roll=0):
+    """Covert an Earth centered fixed sky coordinates to Earth centered rotating.
 
     Args:
-        albedo: `float`, The ratio of reflected light to incident light
-            off of the object's surface
-        distance: `float`, distance to object in meters
-        radius: `float`, radius of sphere in meters
-        phase_angle: `float`, the angle between observer, object, and sun in degrees
+        ra: `float`, right ascension in degrees
+        dec: `float`, declination in degrees
+        roll: `float`, field rotation (ignored)
 
     Returns:
-        A `float`, calculated visual magnitude
+        A `float`, ra, dec and roll in degrees
     """
-    phase_angle = np.deg2rad(phase_angle)
+    sc = SkyCoord(ra=ra, dec=dec, unit='deg', frame=ICRS)
+    sc2 = sc.transform_to(ITRS(obstime=Time(time)))
+    sc3 = SkyCoord(sc2)
 
-    mv_sun = -26.74
+    _, dec, ra = cartesian_to_spherical(sc3.x, sc3.y, sc3.z)
+    dec = dec + 270 * u.deg
+    ra = -ra
 
-    # Lambertian sphere approximation.
-    phase_factor = np.sin(phase_angle) + (np.pi - phase_angle) * np.cos(phase_angle)
-    intensity = phase_factor * (2 * albedo * (radius * radius)) / (3 * np.pi * (distance * distance))
+    return ra, dec, roll
 
-    # Convert intensities to magnitudes
-    mvVector = mv_sun - 2.5 * np.log10(intensity)
 
-    return mvVector
+def eci_to_radec(x, y, z):
+    """Covert ECI coordinates to ra, dec, and distance.
+
+    Args:
+        x: `float`, x coordinate
+        y: `float`, y coordinate
+        z: `float`, z coordinate
+
+    Returns:
+        A `float`, ra, dec and distance in degrees
+    """
+
+    rho2 = x * x + y * y
+    rho = np.sqrt(rho2)
+
+    r = np.sqrt(rho2 + z * z)
+    dec = 0.0 if r == 0.0 else np.arctan2(z, rho)
+    ra = 0.0 if rho == 0.0 else np.arctan2(y, x)
+    if ra < 0.0:
+        ra += np.pi * 2
+
+    return ra * 180 / np.pi, dec * 180 / np.pi, r
+
+
+def radec_to_eci(ra, dec, d=0):
+    """Covert ra, dec, and distance to ECI coordinates.
+
+    Args:
+        ra: `float`, right ascension in degrees
+        dec: `float`, declination in degrees
+        d: `float`, distance
+
+    Returns:
+        A `float`, x, y, and z coordinates
+    """
+
+    ra = ra * np.pi / 180
+    dec = dec * np.pi / 180
+
+    r = 1.0 if d is None or d <= 0.0 else d
+
+    rCosTheta = r * np.cos(dec)
+
+    x = rCosTheta * np.cos(ra)
+    y = rCosTheta * np.sin(ra)
+    z = r * np.sin(dec)
+
+    return x, y, z
